"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const RunAction_1 = require("../../lib/actions/RunAction");
const Constants_1 = require("../../Constants");
const CodeAnalyzerConfigFactory_1 = require("../../lib/factories/CodeAnalyzerConfigFactory");
const EnginePluginsFactory_1 = require("../../lib/factories/EnginePluginsFactory");
const ResultsWriter_1 = require("../../lib/writers/ResultsWriter");
const ResultsViewer_1 = require("../../lib/viewers/ResultsViewer");
const RunSummaryViewer_1 = require("../../lib/viewers/RunSummaryViewer");
const messages_1 = require("../../lib/messages");
const LogEventListener_1 = require("../../lib/listeners/LogEventListener");
const ProgressEventListener_1 = require("../../lib/listeners/ProgressEventListener");
const Display_1 = require("../../lib/Display");
class RunCommand extends sf_plugins_core_1.SfCommand {
    // We don't need the `--json` output for this command.
    static enableJsonFlag = false;
    static summary = (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'command.summary');
    static description = (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'command.description');
    static examples = (0, messages_1.getMessages)(messages_1.BundleName.RunCommand, 'command.examples');
    // TODO: Update when we go to Beta and when we go GA
    static state = (0, messages_1.getMessage)(messages_1.BundleName.Shared, 'label.command-state');
    static flags = {
        // === Flags pertaining to targeting ===
        workspace: sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.workspace.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.workspace.description'),
            char: 'w',
            multiple: true,
            delimiter: ',',
            default: ['.']
        }),
        'path-start': sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.path-start.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.path-start.description'),
            char: 's',
            multiple: true,
            delimiter: ',',
            hidden: true
        }),
        // === Flags pertaining to rule selection ===
        'rule-selector': sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.rule-selector.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.rule-selector.description'),
            char: 'r',
            multiple: true,
            delimiter: ',',
            default: ["Recommended"]
        }),
        // === Flags pertaining to output ===
        'severity-threshold': sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.severity-threshold.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.severity-threshold.description'),
            char: 't'
        }),
        view: sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.view.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.view.description'),
            char: 'v',
            default: Constants_1.View.TABLE,
            options: Object.values(Constants_1.View)
        }),
        'output-file': sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.output-file.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.output-file.description'),
            char: 'f',
            multiple: true,
            delimiter: ','
        }),
        // === Flags pertaining to configuration ===
        'config-file': sf_plugins_core_1.Flags.file({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.config-file.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'flags.config-file.description'),
            char: 'c',
            exists: true
        })
    };
    async run() {
        // TODO: Update when we go to Beta and when we go GA
        this.warn((0, messages_1.getMessage)(messages_1.BundleName.Shared, "warning.command-state", [(0, messages_1.getMessage)(messages_1.BundleName.Shared, 'label.command-state')]));
        const parsedFlags = (await this.parse(RunCommand)).flags;
        const dependencies = this.createDependencies(parsedFlags.view, parsedFlags['output-file']);
        const action = RunAction_1.RunAction.createAction(dependencies);
        const runInput = {
            'config-file': parsedFlags['config-file'],
            'output-file': parsedFlags['output-file'] ?? [],
            'path-start': parsedFlags['path-start'], // TODO: We should move validation of this here instead of having it in the RunAction.
            'rule-selector': parsedFlags['rule-selector'],
            'workspace': parsedFlags['workspace'],
            'severity-threshold': parsedFlags['severity-threshold'] === undefined ? undefined :
                convertThresholdToEnum(parsedFlags['severity-threshold'].toLowerCase())
        };
        await action.execute(runInput);
    }
    createDependencies(view, outputFiles = []) {
        const uxDisplay = new Display_1.UxDisplay(this, this.spinner);
        return {
            configFactory: new CodeAnalyzerConfigFactory_1.CodeAnalyzerConfigFactoryImpl(),
            pluginsFactory: new EnginePluginsFactory_1.EnginePluginsFactoryImpl(),
            writer: ResultsWriter_1.CompositeResultsWriter.fromFiles(outputFiles),
            logEventListeners: [new LogEventListener_1.LogEventDisplayer(uxDisplay)],
            progressListeners: [new ProgressEventListener_1.EngineRunProgressSpinner(uxDisplay), new ProgressEventListener_1.RuleSelectionProgressSpinner(uxDisplay)],
            resultsViewer: view === Constants_1.View.TABLE
                ? new ResultsViewer_1.ResultsTableDisplayer(uxDisplay)
                : new ResultsViewer_1.ResultsDetailDisplayer(uxDisplay),
            runSummaryViewer: new RunSummaryViewer_1.RunSummaryDisplayer(uxDisplay)
        };
    }
}
exports.default = RunCommand;
function convertThresholdToEnum(threshold) {
    // We could do all sorts of complicated conversion logic, but honestly it's just easier
    // to do a switch-statement.
    switch (threshold) {
        case '1':
        case 'critical':
            return code_analyzer_core_1.SeverityLevel.Critical;
        case '2':
        case 'high':
            return code_analyzer_core_1.SeverityLevel.High;
        case '3':
        case 'moderate':
            return code_analyzer_core_1.SeverityLevel.Moderate;
        case '4':
        case 'low':
            return code_analyzer_core_1.SeverityLevel.Low;
        case '5':
        case 'info':
            return code_analyzer_core_1.SeverityLevel.Info;
        default:
            throw new Error((0, messages_1.getMessage)(messages_1.BundleName.RunCommand, 'error.invalid-severity-threshold', [threshold, JSON.stringify(Object.values(code_analyzer_core_1.SeverityLevel))]));
    }
}
//# sourceMappingURL=run.js.map