import { Ux, Spinner } from '@salesforce/sf-plugins-core';
/**
 * Interface for objects that display output information to users. E.g., a class that prints to the CLI would implement
 * this interface.
 * Contrast with a {@code Viewer} implementation, e.g. {@link RuleViewer}, which is responsible for arranging data and
 * handing it off to an underlying {@code Display} implementation.
 */
export interface Display {
    /**
     * Outputs message to stdout at error-level (non-blocking) only if the "--json" flag is not present.
     */
    displayError(msg: string): void;
    /**
     * Output message to stdout at warning-level (non-blocking) only if the "--json" flag is not present.
     */
    displayWarning(msg: string): void;
    /**
     * Output message to stdout at info-level (non-blocking) only if the "--json" flag is not present.
     */
    displayInfo(msg: string): void;
    /**
     * Output message to stdout at log-level (non-blocking) only if the "--json" flag is not present.
     */
    displayLog(msg: string): void;
    /**
     * Output table to stdout only if the "--json" flag is not present.
     */
    displayTable<R extends Ux.Table.Data>(data: R[], columns: Ux.Table.Columns<R>): void;
    /**
     * Prompt the user to confirm that the described action should be carried out, and block until they respond (or a timeout
     * elapses). Resolves to {@code true} if the user confirms, or {@code false} if they don't.
     */
    confirm(message: string): Promise<boolean>;
    spinnerStart(msg: string, status?: string): void;
    spinnerUpdate(status: string): void;
    spinnerStop(status: string): void;
}
export declare class UxDisplay implements Display {
    private readonly displayable;
    private readonly spinner;
    constructor(displayable: Displayable, spinner: Spinner);
    displayError(msg: string): void;
    displayWarning(msg: string): void;
    displayInfo(message: string): void;
    displayLog(message: string): void;
    displayTable<R extends Ux.Table.Data>(data: R[], columns: Ux.Table.Columns<R>): void;
    confirm(message: string): Promise<boolean>;
    spinnerStart(msg: string, status?: string): void;
    spinnerUpdate(status: string): void;
    spinnerStop(status: string): void;
}
export interface Displayable {
    /**
     * Output message to stdout at error-level (non-blocking) only when "--json" flag is not present. [Implemented by Command]
     * @param options.exit If true, the message will be thrown as an error instead of logged. Default value = true.
     */
    error(message: string, options: {
        exit: boolean;
    }): void;
    warn(message: string): void;
    info(message: string): void;
    log(message?: string): void;
    confirm(message: string): Promise<boolean>;
    table<R extends Ux.Table.Data>(data: R[], columns: Ux.Table.Columns<R>, options?: Ux.Table.Options): void;
}
