"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunAction = void 0;
const core_1 = require("@salesforce/core");
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const PathStartUtil_1 = require("../utils/PathStartUtil");
const WorkspaceUtil_1 = require("../utils/WorkspaceUtil");
const LogWriter_1 = require("../writers/LogWriter");
const LogEventListener_1 = require("../listeners/LogEventListener");
const messages_1 = require("../messages");
class RunAction {
    dependencies;
    constructor(dependencies) {
        this.dependencies = dependencies;
    }
    async execute(input) {
        const config = this.dependencies.configFactory.create(input['config-file']);
        const logWriter = await LogWriter_1.LogFileWriter.fromConfig(config);
        // We always add a Logger Listener to the appropriate listeners list, because we should Always Be Logging.
        this.dependencies.logEventListeners.push(new LogEventListener_1.LogEventLogger(logWriter));
        const core = new code_analyzer_core_1.CodeAnalyzer(config);
        // LogEventListeners should start listening as soon as the Core is instantiated, since Core can start emitting
        // events they listen for basically immediately.
        this.dependencies.logEventListeners.forEach(listener => listener.listen(core));
        const enginePlugins = this.dependencies.pluginsFactory.create();
        const enginePluginModules = config.getCustomEnginePluginModules();
        const addEnginePromises = [
            ...enginePlugins.map(enginePlugin => core.addEnginePlugin(enginePlugin)),
            ...enginePluginModules.map(pluginModule => core.dynamicallyAddEnginePlugin(pluginModule))
        ];
        await Promise.all(addEnginePromises);
        const workspace = await (0, WorkspaceUtil_1.createWorkspace)(core, input.workspace);
        // EngineProgressListeners should start listening right before we call Core's `.selectRules()` method, since
        // that's when progress events can start being emitted.
        this.dependencies.progressListeners.forEach(listener => listener.listen(core));
        const ruleSelection = await core.selectRules(input['rule-selector'], { workspace });
        const runOptions = {
            workspace,
            pathStartPoints: await (0, PathStartUtil_1.createPathStarts)(input['path-start'])
        };
        const results = await core.run(ruleSelection, runOptions);
        // After Core is done running, the listeners need to be told to stop, since some of them have persistent UI elements
        // or file handlers that must be gracefully ended.
        this.dependencies.progressListeners.forEach(listener => listener.stopListening());
        this.dependencies.logEventListeners.forEach(listener => listener.stopListening());
        this.dependencies.writer.write(results);
        this.dependencies.resultsViewer.view(results);
        this.dependencies.runSummaryViewer.view(results, logWriter.getLogDestination(), input['output-file']);
        const thresholdValue = input['severity-threshold'];
        if (thresholdValue) {
            throwErrorIfSevThresholdExceeded(thresholdValue, results);
        }
    }
    static createAction(dependencies) {
        return new RunAction(dependencies);
    }
}
exports.RunAction = RunAction;
function throwErrorIfSevThresholdExceeded(threshold, results) {
    let exceedingCount = 0;
    let mostIntenseSeverity = Number.MAX_SAFE_INTEGER;
    for (let i = threshold; i > 0; i--) {
        const sevCount = results.getViolationCountOfSeverity(i);
        if (sevCount > 0) {
            exceedingCount += sevCount;
            mostIntenseSeverity = i;
        }
    }
    if (exceedingCount > 0) {
        const message = (0, messages_1.getMessage)(messages_1.BundleName.RunAction, 'error.severity-threshold-exceeded', [exceedingCount, code_analyzer_core_1.SeverityLevel[threshold]]);
        // Use an SfError because we can easily set an exit code, and it will know what to do with that.
        throw new core_1.SfError(message, 'ThresholdExceeded', [], mostIntenseSeverity);
    }
}
//# sourceMappingURL=RunAction.js.map