"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnnotatedConfigModel = exports.OutputFormat = void 0;
const js_yaml_1 = require("js-yaml");
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const StylingUtil_1 = require("../utils/StylingUtil");
const messages_1 = require("../messages");
const node_path_1 = __importDefault(require("node:path"));
var OutputFormat;
(function (OutputFormat) {
    OutputFormat["RAW_YAML"] = "RAW_YAML";
    OutputFormat["STYLED_YAML"] = "STYLED_YAML";
})(OutputFormat || (exports.OutputFormat = OutputFormat = {}));
class AnnotatedConfigModel {
    relevantEngines;
    userContext;
    defaultContext;
    constructor(relevantEngines, userContext, defaultContext) {
        this.relevantEngines = relevantEngines;
        this.userContext = userContext;
        this.defaultContext = defaultContext;
    }
    toFormattedOutput(format) {
        // istanbul ignore else: Should be impossible
        if (format === OutputFormat.STYLED_YAML) {
            return new StyledYamlFormatter(this.relevantEngines, this.userContext, this.defaultContext).toYaml();
        }
        else if (format === OutputFormat.RAW_YAML) {
            return new PlainYamlFormatter(this.relevantEngines, this.userContext, this.defaultContext).toYaml();
        }
        else {
            throw new Error(`Unsupported`);
        }
    }
    static fromSelection(relevantEngines, userContext, defaultContext) {
        return new AnnotatedConfigModel(relevantEngines, userContext, defaultContext);
    }
}
exports.AnnotatedConfigModel = AnnotatedConfigModel;
class YamlFormatter {
    relevantEngines;
    userContext;
    defaultContext;
    constructor(relevantEngines, userContext, defaultContext) {
        this.relevantEngines = relevantEngines;
        this.userContext = userContext;
        this.defaultContext = defaultContext;
    }
    toYamlSectionHeadingComment(commentText) {
        const horizontalLine = '='.repeat(70);
        return this.toYamlComment(`${horizontalLine}\n${commentText}\n${horizontalLine}`);
    }
    toYamlUncheckedField(fieldName, fieldValue) {
        return (0, js_yaml_1.dump)({ [fieldName]: fieldValue }).trim();
    }
    toYamlUncheckedFieldWithInlineComment(fieldName, fieldValue, commentText) {
        const yamlCode = this.toYamlUncheckedField(fieldName, fieldValue);
        const comment = this.toYamlComment(commentText);
        return yamlCode.replace(/(\r?\n|$)/, ` ${comment}$1`);
    }
    toYamlField(fieldName, userValue, defaultValue) {
        if (looksLikeAPathValue(userValue) && userValue === defaultValue) {
            // We special handle a path value when it is equal to the default value, making it equal null because
            // chances are it is a derived file or folder value based on the specific environment that we do not want to
            // actually want to hard code since checking in the config to CI/CD system may create a different value
            const commentText = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.last-calculated-as', [JSON.stringify(userValue)]);
            return this.toYamlUncheckedFieldWithInlineComment(fieldName, null, commentText);
        }
        else if (JSON.stringify(userValue) === JSON.stringify(defaultValue)) {
            return this.toYamlUncheckedField(fieldName, userValue);
        }
        userValue = replaceAbsolutePathsWithRelativePathsWherePossible(userValue, this.userContext.config.getConfigRoot() + node_path_1.default.sep);
        const commentText = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.modified-from', [JSON.stringify(defaultValue)]);
        return this.toYamlUncheckedFieldWithInlineComment(fieldName, userValue, commentText);
    }
    toYaml() {
        const topLevelDescription = code_analyzer_core_1.CodeAnalyzerConfig.getConfigDescription();
        return this.toYamlSectionHeadingComment(topLevelDescription.overview) + '\n' +
            '\n' +
            this.toYamlComment(topLevelDescription.fieldDescriptions.config_root) + '\n' +
            this.toYamlField('config_root', this.userContext.config.getConfigRoot(), this.defaultContext.config.getConfigRoot()) + '\n' +
            '\n' +
            this.toYamlComment(topLevelDescription.fieldDescriptions.log_folder) + '\n' +
            this.toYamlField('log_folder', this.userContext.config.getLogFolder(), this.defaultContext.config.getLogFolder()) + '\n' +
            '\n' +
            this.toYamlComment(topLevelDescription.fieldDescriptions.rules) + '\n' +
            this.toYamlRuleOverrides() + '\n' +
            '\n' +
            this.toYamlComment(topLevelDescription.fieldDescriptions.engines) + '\n' +
            this.toYamlEngineOverrides() + '\n' +
            '\n' +
            this.toYamlSectionHeadingComment((0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.common.end-of-config')) + '\n';
    }
    toYamlRuleOverrides() {
        if (this.userContext.rules.getCount() === 0) {
            const commentText = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.yaml.no-rules-selected');
            return `rules: {} ${this.toYamlComment(commentText)}`;
        }
        let yamlCode = 'rules:\n';
        for (const engineName of this.userContext.rules.getEngineNames()) {
            yamlCode += '\n';
            yamlCode += (0, StylingUtil_1.indent)(this.toYamlRuleOverridesForEngine(engineName), 2) + '\n';
        }
        return yamlCode.trimEnd();
    }
    toYamlRuleOverridesForEngine(engineName) {
        const engineConfigHeader = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.rule-overrides-section', [engineName.toUpperCase()]);
        let yamlCode = this.toYamlSectionHeadingComment(engineConfigHeader) + '\n';
        yamlCode += `${engineName}:\n`;
        for (const userRule of this.userContext.rules.getRulesFor(engineName)) {
            const defaultRule = this.getDefaultRuleFor(engineName, userRule.getName());
            yamlCode += (0, StylingUtil_1.indent)(this.toYamlRuleOverridesForRule(userRule, defaultRule), 2) + '\n';
        }
        return yamlCode.trimEnd();
    }
    getDefaultRuleFor(engineName, ruleName) {
        try {
            return this.defaultContext.rules.getRule(engineName, ruleName);
        }
        catch (e) {
            // istanbul ignore next
            return null;
        }
    }
    toYamlRuleOverridesForRule(userRule, defaultRule) {
        const userSeverity = userRule.getSeverityLevel();
        const userTags = userRule.getTags();
        return `"${userRule.getName()}":\n` +
            (0, StylingUtil_1.indent)(this.toYamlField('severity', userSeverity, defaultRule !== null ? defaultRule.getSeverityLevel() : userSeverity), 2) + '\n' +
            (0, StylingUtil_1.indent)(this.toYamlField('tags', userTags, defaultRule !== null ? defaultRule.getTags() : userTags), 2);
    }
    toYamlEngineOverrides() {
        if (this.relevantEngines.size === 0) {
            const commentText = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.yaml.no-engines-selected');
            return `engines: {} ${this.toYamlComment(commentText)}`;
        }
        let yamlCode = 'engines:\n';
        for (const engineName of this.relevantEngines.keys()) {
            yamlCode += (0, StylingUtil_1.indent)(this.toYamlEngineOverridesForEngine(engineName), 2) + '\n';
        }
        return yamlCode.trimEnd();
    }
    toYamlEngineOverridesForEngine(engineName) {
        const engineConfigDescriptor = this.userContext.core.getEngineConfigDescription(engineName);
        const userEngineConfig = this.userContext.core.getEngineConfig(engineName);
        const defaultEngineConfig = this.defaultContext.core.getEngineConfig(engineName);
        let yamlCode = '\n' +
            this.toYamlSectionHeadingComment(engineConfigDescriptor.overview) + '\n' +
            `${engineName}:\n`;
        // By fiat, the field description will always include, at minimum, an entry for "disable_engine", so we can
        // assume that the object is not undefined.
        for (const configField of Object.keys(engineConfigDescriptor.fieldDescriptions)) {
            const fieldDescription = engineConfigDescriptor.fieldDescriptions[configField];
            const defaultValue = defaultEngineConfig[configField] ?? null;
            const userValue = userEngineConfig[configField] ?? defaultValue;
            // Add a leading newline to visually break up the property from the previous one.
            yamlCode += '\n' +
                (0, StylingUtil_1.indent)(this.toYamlComment(fieldDescription), 2) + '\n' +
                (0, StylingUtil_1.indent)(this.toYamlField(configField, userValue, defaultValue), 2) + '\n';
        }
        return yamlCode.trimEnd();
    }
}
class PlainYamlFormatter extends YamlFormatter {
    constructor(relevantEngines, userContext, defaultContext) {
        super(relevantEngines, userContext, defaultContext);
    }
    toYamlComment(commentText) {
        return commentText.replace(/^.*/gm, s => `# ${s}`);
    }
}
class StyledYamlFormatter extends YamlFormatter {
    constructor(relevantEngines, userContext, defaultContext) {
        super(relevantEngines, userContext, defaultContext);
    }
    toYamlComment(commentText) {
        return commentText.replace(/^.*/gm, s => (0, StylingUtil_1.makeGrey)(`# ${s}`));
    }
}
function looksLikeAPathValue(value) {
    return typeof (value) === 'string' && !value.includes('\n') && value.includes(node_path_1.default.sep);
}
function replaceAbsolutePathsWithRelativePathsWherePossible(value, parentFolder) {
    if (typeof value === 'string') {
        // Check if the string starts with the parent folder
        if (value.startsWith(parentFolder)) {
            // Strip the parent folder from the start of the string
            return value.substring(parentFolder.length);
        }
        return value; // Return unchanged if it doesn't start with the parent folder
    }
    else if (Array.isArray(value)) {
        // If value is an array, recursively process each element
        return value.map(item => replaceAbsolutePathsWithRelativePathsWherePossible(item, parentFolder));
    }
    else if (typeof value === 'object' && value !== null) {
        // If value is an object, recursively process each key-value pair
        const updatedObject = {};
        for (const key in value) {
            updatedObject[key] = replaceAbsolutePathsWithRelativePathsWherePossible(value[key], parentFolder);
        }
        return updatedObject;
    }
    // Return the value unchanged if it's a number, boolean, or null
    return value;
}
//# sourceMappingURL=ConfigModel.js.map